/*!
* \file joueur.c
* \brief Fichier contenant le code des fonctions liees a la structure Partie.
*/

#include <stdio.h>
#include <stdlib.h>
#include "structures.h"
#include "partie.h"
#include "joueurList.h"
#include "joueur.h"

/*!
 * \fn Partie * initPartie()
 * \brief La fonction cree une nouvelle Partie et initialise ses valeurs.
 *
 * \return Un pointeur vers la Partie creee.
 */
 Partie * initPartie(){
    Partie * p = malloc(sizeof(Partie));
    ListJoueur * l = initJoueurList();
    Carte * carteJeu = NULL;
    p->nbTours=0;
    p->participants=l;
    init_carte(carteJeu,LARG_MAX_CARTE,HAUT_MAX_CARTE);
    p->c=carteJeu;
    return p;
 }

 /*!
 * \fn void deletePartie(Partie * p)
 * \brief La fonction efface une Partie et libere l'espace en memoire.
 *
 * \param La Partie p a effacer
 */
 void deletePartie(Partie * p){
    deleteJoueurList(p->participants);
    free(p);
 }

  /*!
 * \fn int getNbTour(Partie * p)
 * \brief La fonction renvoie le nombre de tours actuel de la Partie.
 *
 * \param La Partie p a tester.
 * \return Le nombre de tour actuel de la Partie, -1 si l'entree est NULL.
 */
int getNbTour(Partie * p){
    if(p!=NULL){
        return p->nbTours;
    }
    return -1;
}

  /*!
 * \fn ListJoueur * getListJoueur(Partie * p)
 * \brief La fonction renvoie un pointeur vers la liste de Joueurs de la Partie.
 *
 * \param La Partie p a tester.
 * \return Un pointeur vers la ListJoueur de la Partie, NULL si l'entree est NULL.
 */
ListJoueur * getListJoueur(Partie * p){
    if(p!=NULL){
        return p->participants;
    }
    return NULL;
}

  /*!
 * \fn Carte * getCarte(Partie * p)
 * \brief La fonction renvoie un pointeur vers la Carte de la Partie.
 *
 * \param La Partie p a tester.
 * \return Un pointeur vers la Carte de la Partie, NULL si l'entree est NULL.
 */
Carte * getCarte(Partie * p){
    if(p!=NULL){
        return p->c;
    }
    return NULL;
}

  /*!
 * \fn void augmenterNbTour(Partie * p)
 * \brief La fonction incremente le nombre de tours de la Partie
 *
 * \param La Partie p a modifier.
 */
 void augmenterNbTour(Partie * p){
    if(p!=NULL){
        p->nbTours++;
    }
 }

   /*!
 * \fn int addParticipant(Partie * p, Joueur * j)
 * \brief La fonction ajoute un Joueur a la Partie.
 *
 * \param La Partie p a a modifier, le Joueur a ajouter.
 * \return 1 si le Joueur a ete correctement ajoute, 0 sinon.
 */
 int addParticipant(Partie * p, Joueur * j){
    if((p!=NULL)&&(j!=NULL)){
        NodeJoueur * nJ = initNodeJoueur(j);
        return addNodeJoueurLast(p->participants,nJ);
    }
    return 0;
 }

   /*!
 * \fn int removeParticipant(Partie * p, Joueur * j)
 * \brief La fonction retire un Joueur de la Partie et sauve ces informations.
 *
 * \param La Partie p a a modifier, le Joueur a retirer et a sauver.
 * \return 1 si le Joueur a ete correctement retire, 0 sinon.
 */
int removeParticipant(Partie * p, Joueur * j){
    /*A Ajouter : une fonction sauvant les informations du joueur a retire.*/
    return deleteNodeJoueurFromList(p->participants,j);
}

   /*!
 * \fn int finTour(Partie * p)
 * \brief La fonction effectue les operations de fin de tour.
 *        On teste si un joueur a gagne la partie.
 *        Le curseur de participant est deplace sur le joueur suivant.
 *        On pourra ajouter d'autre operations de fin de tour dans les prochaines versions.
 *
 * \param La Partie p a a modifier.
 * \return 1 si un Joueur est victorieux, 0 sinon
 */
int finTour(Partie * p){
    if(!victoire(p)){
        if(!outOfJoueurList(p->participants)){
            nextJoueur(p->participants);
            return 0;
        }
        else{
            setOnFirstJoueur(p->participants);
            return 0;
        }
    }
    else{
        return 1;
    }
}

   /*!
 * \fn int isTurn(Partie * p, Joueur * j)
 * \brief La fonction teste si c'est au tour du joueur en parrametre.
 *
 * \param La Partie p a tester, le joueur a tester.
 * \return 1 si c'est le tour du Joueur, 0 sinon.
 */
int isTurn(Partie * p, Joueur * j){
    return(getCurrentJoueur(p->participants)==j);
}

   /*!
 * \fn Personnage * jouerPersonnage(Partie * p, Joueur * j, Personnage * perso)
 * \brief La fonction renvoie le personnage en parrametre s'il appartient au Joueur et que c'est le tour du Joueur.
 *        Cette fonction permet de tester si le Joueur J peut ou non effectuer des action avec un Personnage.
 *
 * \param La Partie p a tester, le Joueur a tester, le Personnage a tester.
 * \return Un pointeur vers le Personnage du Joueur si le Joueur peut jouer ce Personnage, NULL sinon.
 */
Personnage * jouerPersonnage(Partie * p, Joueur * j, Personnage * perso){
    if((isInGroupe(j,perso))&&(isTurn(p,j))){
        return perso;
    }
    return NULL;
}

   /*!
 * \fn int victoire(Partie * p)
 * \brief La fonction teste si la Partie a ete remportee.
 *
 * \param La Partie p a tester.
 * \return 1 si la partie est gagnee, 0 sinon.
 */
int victoire(Partie * p){
    return (p->participants->firstNodeJoueur==p->participants->lastNodeJoueur);
}

   /*!
 * \fn finPartie(Partie * p)
 * \brief La fonction sauve les informations necessaires (Joueur) et supprime la partie.
 *
 * \param La Partie p a modifier
 */
void finPartie(Partie * p){
    /*A ajouter : une fonction sauvant les informations sur le Joueur.*/
    deletePartie(p);
}



